"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var graphql_subscriptions_1 = require("graphql-subscriptions");
var nats_1 = require("nats");
var nats_pubsub_1 = require("../nats-pubsub");
var logger_1 = require("./logger");
require("jest");
var schema = new graphql_1.GraphQLSchema({
    query: new graphql_1.GraphQLObjectType({
        name: 'Query',
        fields: {
            testString: {
                type: graphql_1.GraphQLString,
                resolve: function () {
                    return 'works';
                },
            },
        },
    }),
    subscription: new graphql_1.GraphQLObjectType({
        name: 'Subscription',
        fields: {
            testSubscription: {
                type: graphql_1.GraphQLString,
                resolve: function (root) {
                    return root;
                },
            },
            testFilter: {
                type: graphql_1.GraphQLString,
                resolve: function (_, _a) {
                    var filterBoolean = _a.filterBoolean;
                    return filterBoolean ? 'goodFilter' : 'badFilter';
                },
                args: {
                    filterBoolean: { type: graphql_1.GraphQLBoolean },
                },
            },
            testFilterMulti: {
                type: graphql_1.GraphQLString,
                resolve: function (_, _a) {
                    var filterBoolean = _a.filterBoolean;
                    return filterBoolean ? 'goodFilter' : 'badFilter';
                },
                args: {
                    filterBoolean: { type: graphql_1.GraphQLBoolean },
                    a: { type: graphql_1.GraphQLString },
                    b: { type: graphql_1.GraphQLInt },
                },
            },
            testChannelOptions: {
                type: graphql_1.GraphQLString,
                resolve: function (root) {
                    return root;
                },
                args: {
                    repoName: { type: graphql_1.GraphQLString },
                },
            },
        },
    }),
});
var natsClient = nats_1.connect('nats://localhost:4222');
var subManager = new graphql_subscriptions_1.SubscriptionManager({
    schema: schema,
    setupFunctions: {
        'testFilter': function (_, _a) {
            var filterBoolean = _a.filterBoolean;
            return {
                'Filter1': { filter: function (root) { return root.filterBoolean === filterBoolean; } },
            };
        },
        'testFilterMulti': function () {
            return {
                'Trigger1': { filter: function () { return true; } },
                'Trigger2': { filter: function () { return true; } },
            };
        },
    },
    pubsub: new nats_pubsub_1.NatsPubSub({
        client: natsClient,
        logger: logger_1.logger,
    }),
});
describe('SubscriptionManager', function () {
    var _this = this;
    var originalTimeout;
    beforeAll(function (done) {
        natsClient.on('connect', function () {
            done();
        });
    });
    afterAll(function (done) {
    });
    it('throws an error if query is not valid', function () { return __awaiter(_this, void 0, void 0, function () {
        var query, callback;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    query = 'query a{ testInt }';
                    callback = function () { return null; };
                    return [4, expect(subManager.subscribe({ query: query, operationName: 'a', callback: callback }))
                            .rejects.toMatchObject({ message: 'Subscription query has validation errors' })];
                case 1: return [2, _a.sent()];
            }
        });
    }); });
    it('rejects subscriptions with more than one root field', function () { return __awaiter(_this, void 0, void 0, function () {
        var query, callback;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    query = 'subscription X{ a: testSubscription, b: testSubscription }';
                    callback = function () { return null; };
                    return [4, expect(subManager.subscribe({ query: query, operationName: 'X', callback: callback }))
                            .rejects.toMatchObject({ message: 'Subscription query has validation errors' })];
                case 1: return [2, _a.sent()];
            }
        });
    }); });
    it('can subscribe with a valid query and gets a subId back', function (done) {
        var query = 'subscription X{ testSubscription }';
        var callback = function () { return null; };
        subManager.subscribe({ query: query, operationName: 'X', callback: callback }).then(function (subId) {
            expect(typeof subId).toBe('number');
            logger_1.logger.trace('subid (%s)', subId);
            try {
                subManager.unsubscribe(subId);
                done();
            }
            catch (e) {
                done.fail(e);
            }
        });
    });
    it('can subscribe with a valid query and get the root value', function (done) {
        var query = 'subscription X{ testSubscription}';
        var callback = function (err, payload) {
            if (err) {
                done.fail(err);
            }
            logger_1.logger.trace('testOutput#4, payload (%j)', payload);
            try {
                expect(payload.data.testSubscription).toBe('good');
            }
            catch (e) {
                done.fail(e);
                return;
            }
            done();
        };
        subManager.subscribe({ query: query, operationName: 'X', callback: callback }).then(function (subId) {
            subManager.publish('testSubscription', 'good');
            setTimeout(function () {
                subManager.unsubscribe(subId);
            }, 5);
        });
    });
    it('can use filter functions properly', function (done) {
        var query = "subscription Filter1($filterBoolean: Boolean){\n            testFilter(filterBoolean: $filterBoolean)\n            }";
        var callback = function (err, payload) {
            if (err) {
                done.fail(err);
            }
            try {
                expect(payload.data.testFilter).toEqual('goodFilter');
            }
            catch (e) {
                done.fail(e);
                return;
            }
            done();
        };
        subManager.subscribe({
            query: query,
            operationName: 'Filter1',
            variables: { filterBoolean: true },
            callback: callback,
        }).then(function (subId) {
            subManager.publish('Filter1', { filterBoolean: false });
            subManager.publish('Filter1', { filterBoolean: true });
            setTimeout(function () {
                subManager.unsubscribe(subId);
            }, 20);
        });
    });
    it('can subscribe to more than one trigger', function (done) {
        var triggerCount = 0;
        var query = "subscription multiTrigger($filterBoolean: Boolean, $uga: String){\n       testFilterMulti(filterBoolean: $filterBoolean, a: $uga, b: 66)\n      }";
        var callback = function (err, payload) {
            expect(payload.data.testFilterMulti).toBe('goodFilter');
            triggerCount++;
            if (triggerCount === 2) {
                done();
            }
        };
        subManager.subscribe({
            query: query,
            operationName: 'multiTrigger',
            variables: { filterBoolean: true, uga: 'UGA' },
            callback: callback,
        }).then(function (subId) {
            subManager.publish('NotATrigger', { filterBoolean: false });
            subManager.publish('Trigger1', { filterBoolean: true });
            subManager.publish('Trigger2', { filterBoolean: true });
            setTimeout(function () {
                subManager.unsubscribe(subId);
            }, 6);
        });
    });
    it('can unsubscribe', function (done) {
        var query = 'subscription X{ testSubscription }';
        var callback = function (err, payload) {
            done();
        };
        subManager.subscribe({ query: query, operationName: 'X', callback: callback }).then(function (subId) {
            subManager.unsubscribe(subId);
            subManager.publish('testSubscription', 'bad');
            setTimeout(done, 1000);
        });
    });
    it('throws an error when trying to unsubscribe from unknown id', function () {
        expect(function () { return subManager.unsubscribe(123); }).toThrowError('undefined');
    });
    it('calls the error callback if there is an execution error', function (done) { return __awaiter(_this, void 0, void 0, function () {
        var query, callback;
        return __generator(this, function (_a) {
            query = "subscription X($uga: Boolean!){\n      testSubscription  @skip(if: $uga)\n    }";
            callback = function (err, payload) {
                return __awaiter(this, void 0, void 0, function () {
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                logger_1.logger.trace('test# received err (%j), payload (%j) ', err, payload);
                                return [4, expect(payload).toBeUndefined()];
                            case 1:
                                _a.sent();
                                return [4, expect(err.message).toMatchObject({ GraphqlError: 'Variable "$uga" of required type "Boolean!" was not provided.' })];
                            case 2:
                                _a.sent();
                                done();
                                return [2];
                        }
                    });
                });
            };
            subManager.subscribe({ query: query, operationName: 'X', callback: callback }).then(function (subId) {
                subManager.publish('testSubscription', 'good');
                setTimeout(function () {
                    subManager.unsubscribe(subId);
                }, 40);
            });
            return [2];
        });
    }); });
    it('can use transform function to convert the trigger name given into more explicit channel name', function (done) {
        var triggerTransform = function (trigger, _a) {
            var path = _a.path;
            return [trigger].concat(path).join('.');
        };
        var pubsub = new nats_pubsub_1.NatsPubSub({
            triggerTransform: triggerTransform,
            logger: logger_1.logger,
        });
        var subManager2 = new graphql_subscriptions_1.SubscriptionManager({
            schema: schema,
            setupFunctions: {
                testChannelOptions: function (options, _a) {
                    var repoName = _a.repoName;
                    return ({
                        comments: {
                            channelOptions: { path: [repoName] },
                        },
                    });
                },
            },
            pubsub: pubsub,
        });
        var callback = function (err, payload) {
            expect(payload.data.testChannelOptions).toBe('test');
            done();
        };
        var query = "\n      subscription X($repoName: String!) {\n        testChannelOptions(repoName: $repoName)\n      }\n    ";
        var variables = { repoName: 'graphql-nats-subscriptions' };
        subManager2.subscribe({ query: query, operationName: 'X', variables: variables, callback: callback }).then(function (subId) {
            pubsub.publish('comments.graphql-nats-subscriptions', 'test');
            setTimeout(function () { return pubsub.unsubscribe(subId); }, 40);
        });
    });
});
//# sourceMappingURL=integration.test.js.map